/**
 * Created by Grigorii Shovkoplias on 22.01.2016
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        new File("../preliminary").mkdir();
        desc = new PrintWriter("../tests.desc");


        done = 0;
        genSamples();

        //-------------------------first group-------------------------------------------------
        subtask = "subtask1";

        for (int i = 0; i < 3; i++) {
            genRand1(10);
            genRand11(5, 50);
        }

        for (int i = 0; i < 7; i++) {
            genRand11(100, MAXN);
            genRand1(MAXN);
        }

        desc.println(String.format("comment_subtask%d=\"h mod a = w mod b = 0 \"", 1));
        desc.println(String.format("bonus_subtask%d=%d", 1, 50));

        //-------------------------second group-------------------------------------------------
        subtask = "subtask2";

        genSimples();
        for (int i = 0; i < 3; i++) {
            genRand(10);
            genRandSmall(5, 50);
        }

        for (int i = 0; i < 7; i++) {
            genRand(MAXN);
            genRandSmall(100, MAXN);
        }

        desc.println(String.format("comment_subtask%d=\"full limits\"", 2));
        desc.println(String.format("bonus_subtask%d=%d", 2, 50));


        desc.close();

    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private PrintWriter desc;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN = 1000;



    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private int gand(int n) {
        return rand(n / 2) + n / 2 + 1;
    }

    private char zeroOrOne() {
        return (char) ('0' + rnd.nextInt(2));
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }

    private void print(int a, int b, int c, int d) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(a + " " + b + " " + c + " " + d);
        out.close();
    }


    private String getPath() {
        return String.format("../tests/%02d", done);
    }

    private void printDesc(String path, String folder, String subtask, int points, String comment) throws IOException {
        desc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, points, comment));
    }

    private String subtask;

    private void printDescT(String comment) throws IOException {
        printDesc(getPath(), "tests", subtask, 0, comment);
    }

    //begin generators

    private void genSamples() throws IOException {
        PrintWriter out;
        String path;
        path = "../preliminary/01";
        out = new PrintWriter(path);
        out.println("1 1 3 3");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 1");
        path = "../preliminary/02";
        out = new PrintWriter(path);
        out.println("2 1 3 2");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 2");
        path = "../preliminary/03";
        out = new PrintWriter(path);
        out.println("5 6 1 1");
        out.close();
        printDesc(path, "preliminary", "subtask0", 0, "sample 3");
    }

    private void shuffle(int[] a) {
        for (int i = 0; i < a.length; i++) {
            int j = rand(a.length);
            int tmp = a[j];
            a[j] = a[i];
            a[i] = tmp;
        }
    }
    //begin generators

    private void genRand1(int max) throws IOException {
        int a = rand(max) + 1;
        int b = rand(max) + 1;
        int c = a * (rand(max / a) + 1);
        int d = b * (rand(max / b) + 1);
        print(a, b, c, d);
        printDescT(String.format("Random multiples: %d x %d, %d x %d", a, b, c, d));
    }

    private void genRand11(int max1, int max) throws IOException {
        int a = rand(max1) + 1;
        int b = rand(max1) + 1;
        int c = a * (rand(max / a) + 1);
        int d = b * (rand(max / b) + 1);
        print(a, b, c, d);
        printDescT(String.format("Random small multiples: %d x %d, %d x %d", a, b, c, d));
    }

    private void genRand(int max) throws IOException {
        int a = rand(max) + 1;
        int b = rand(max) + 1;
        int c = gand(max) + 1;
        int d = gand(max) + 1;
        print(a, b, c, d);
        printDescT(String.format("Rand: %d x %d, %d x %d", a, b, c, d));
    }

    private void genRandSmall(int max1, int max) throws IOException {
        int a = rand(max1) + 1;
        int b = rand(max1) + 1;
        int c = gand(max) + 1;
        int d = gand(max) + 1;
        print(a, b, c, d);
        printDescT(String.format("Rand small: %d x %d, %d x %d", a, b, c, d));
    }

    private void genSimples() throws IOException {
        print(1, 2, 2, 1);
        printDescT(String.format("simple 1"));
        print(10, 10, 99, 99);
        printDescT(String.format("simple 2"));
        print(1, 5, 1, 7);
        printDescT(String.format("simple 3"));
        print(1, 1, 1000, 1000);
        printDescT(String.format("simple 4"));
    }



    //end generators

}
